# MIT License
#
# Copyright (c) 2021 Clivern
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import uuid
import json
from .lang import Lang


class Code():
    """Code To Run"""

    def __init__(self, code, lang, version, id=None, meta = {}):
        self._code = code
        self._lang = lang
        self._version = version
        self._id = str(uuid.uuid4()) if id is None else id
        self._meta = meta

    @property
    def code(self):
        """
        Gets the code snippet

        Returns:
            The code snippet
        """
        return self._code

    @property
    def lang(self):
        """
        Gets the language type

        Returns:
            The language type
        """
        return self._lang

    @property
    def version(self):
        """
        Gets the language version

        Returns:
            The language version
        """
        return self._version

    @property
    def id(self):
        """
        Gets the code instance ID

        Returns:
            The code instance ID
        """
        return self._id

    @property
    def meta(self):
        """
        Gets the code meta data

        Returns:
            The code meta data
        """
        return self._meta

    @classmethod
    def from_string(cls, data):
        """
        Get Code from JSON string

        Args:
            data: the JSON string

        Returns:
            An instance of this class
        """
        data = json.loads(data)

        return cls(
            data['code'],
            Lang.get_item(data['lang']),
            data['version'],
            data['id'],
            data['meta']
        )

    def __str__(self):
        """
        Convert the Object to string

        Returns:
            A JSON representation of this instance
        """
        return json.dumps({
            'id': self._id,
            'code': self._code,
            'lang': self._lang.value,
            'version': self._version,
            'meta': self._meta,
        })



# MIT License
#
# Copyright (c) 2021 Clivern
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from enum import Enum


class Lang(Enum):
    """Language Types"""

    RUST = "rust"
    GO = "go"
    PHP = "php"
    PYTHON = "python"
    JAVA = "java"
    RUBY = "ruby"
    C = "c"
    CPLUS = "cplus"

    @classmethod
    def get_item(cls, key):
        """
        Get Enum key with the value

        Args:
            key: the value

        Returns:
            The key
        """
        if key == "rust":
            return cls.RUST

        elif key == "go":
            return cls.GO

        elif key == "php":
            return cls.PHP

        elif key == "python":
            return cls.PYTHON

        elif key == "java":
            return cls.JAVA

        elif key == "ruby":
            return cls.RUBY

        elif key == "c":
            return cls.C

        elif key == "cplus":
            return cls.CPLUS



# MIT License
#
# Copyright (c) 2021 Clivern
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class Cplus():
    """C++ Runtime Class"""

    # Docker Image
    _image = "gcc"

    # Default Version
    _version = "1.17"

    # All supported versions
    _versions = {
        "9.4.0": "GCC Version 9.4.0",
    }

    # File extension
    _extension = "cpp"

    def __init__(self, version="9.4.0"):
        """Class Constructor"""
        self._version = version

    @property
    def script(self):
        """
        Get execution script content

        Returns:
            the execution script content
        """
        return "\n".join([
            "#!/bin/bash",
            "",
            "mkdir -p /tmp",
            "cp -r /code/* /tmp/",
            "cd /tmp",
            "start_time1=$(date +%s%N)",
            "g++ -o run run.cpp",
            "elapsed1=$((($(date +%s%N) - $start_time1)/1000000))",
            "start_time2=$(date +%s%N)",
            "./run",
            "elapsed2=$((($(date +%s%N) - $start_time2)/1000000))",
            "echo \"-------\"",
            "echo \"Build time in milliseconds: \"$elapsed1",
            "echo \"Execution time in milliseconds: \"$elapsed2",
            "",
        ])

    @property
    def versions(self):
        """
        Get all supported versions

        Returns:
            A dict of supported versions
        """
        return self._versions

    @property
    def image(self):
        """
        Get docker image name

        Returns:
            the docker image
        """
        return self._image

    @property
    def version(self):
        """
        Get the default version

        Returns:
            the default version
        """
        return self._version

    @property
    def extension(self):
        """
        Get the extension

        Returns:
            the extension
        """
        return self._extension



# MIT License
#
# Copyright (c) 2021 Clivern
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pindo.runner import Runner

if __name__ == "__main__":

    code = """
    public class HelloWorld{

         public static void main(String []args){
            System.out.println("Hello World");
         }
    }
    """

    java_code = Runner.java(code, "17.0", None, {"main_class": "HelloWorld"})

    engine = Runner.docker(java_code)

    engine.setup()
    print(engine.run())
    engine.cleanup()



# MIT License
#
# Copyright (c) 2021 Clivern
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import stat
import docker
import shutil
from .c import C
from .go import Go
from .php import PHP
from .rust import Rust
from .ruby import Ruby
from .java import Java
from .cplus import Cplus
from .python import Python
from pindo.lang import Lang
from pindo.exception.code_failed_to_run import CodeFailedToRun


class Engine():
    """Docker Engine"""

    def __init__(self, code, docker_client=None):
        self._code = code
        self._runtime = Engine.get_runtime(code)
        self._docker_client = docker.from_env() if docker_client is None else docker_client

    def run(self, configs={}):
        """
        Execute the code on docker

        Returns:
            The output, execution and build time
        """
        defaults = {
            "mount_mode": "ro",
            "mem_limit": "100m",
            "cpu_period": 100000,
            "cpu_quota": 50000
        }

        defaults.update(configs)

        try:
            if "runtime" in defaults.keys():
                # Run container in background using a different runtime
                # like gvisor https://github.com/google/gvisor for better isolation
                container = self._docker_client.containers.run(
                    "{}:{}".format(self._runtime.image, self._runtime.version),
                    "bash /code/exec.sh",
                    name=self._code.id,
                    volumes={self._code.id: {'bind': '/code', 'mode': defaults["mount_mode"]}},
                    detach=True,
                    mem_limit=defaults["mem_limit"],
                    cpu_period=defaults["cpu_period"],
                    cpu_quota=defaults["cpu_quota"],
                    runtime=defaults["runtime"]
                )
            else:
                # Run container in background using the default runtime
                container = self._docker_client.containers.run(
                    "{}:{}".format(self._runtime.image, self._runtime.version),
                    "bash /code/exec.sh",
                    name=self._code.id,
                    volumes={self._code.id: {'bind': '/code', 'mode': defaults["mount_mode"]}},
                    detach=True,
                    mem_limit=defaults["mem_limit"],
                    cpu_period=defaults["cpu_period"],
                    cpu_quota=defaults["cpu_quota"]
                )
        except Exception as e:
            raise CodeFailedToRun("Code %s failed to run: {}".format(self._code.id, str(e)))

        result = []

        for line in container.logs(stream=True):
            result.append(str(line, 'utf-8'))

        result = "".join(result)
        items = result.rsplit("-------", 1)
        stats = items[1].split("\n")
        build_time = None
        execution_time = None

        for x in range(len(stats)):
            if "Build time in milliseconds: " in stats[x]:
                build_time = stats[x].replace("Build time in milliseconds: ", "")
            if "Execution time in milliseconds: " in stats[x]:
                execution_time = stats[x].replace("Execution time in milliseconds: ", "")

        return {
            "output": items[0],
            "build_time": build_time,
            "execution_time": execution_time,
        }

    def setup(self):
        """
        Create an executable script on a docker volume
        """
        try:
            volume = self._docker_client.volumes.create(
                name=self._code.id,
                driver='local'
            )

            file = "{}/{}".format(volume.attrs['Mountpoint'], "exec.sh")

            if self._code.lang == Lang.JAVA:
                # Java Script
                script = "{}/{}.{}".format(
                    volume.attrs['Mountpoint'],
                    self._runtime.main_class,
                    self._runtime.extension
                )
            else:
                # Other Scripts
                script = "{}/run.{}".format(
                    volume.attrs['Mountpoint'],
                    self._runtime.extension
                )

            f = open(file, "w")
            f.write(self._runtime.script)
            f.close()

            f = open(script, "w")
            f.write(self._code.code)
            f.close()

        except Exception as e:
            raise Exception("Error while creating volume: {}".format(str(e)))

    def cleanup(self):
        """
        Removes the code volume
        """
        try:
            container = self._docker_client.containers.get(self._code.id)

            if container:
                self._docker_client.api.remove_container(container.id, force=True, v=True)

            volume = self._docker_client.volumes.get(self._code.id)

            if volume:
                volume.remove(force=True)

        except Exception as e:
            raise Exception("Error while removing the volume: {}".format(str(e)))

    @property
    def code(self):
        """
        Returns:
            The code instance
        """
        return self._code

    @property
    def runtime(self):
        """
        Returns:
            The runtime instance
        """
        return self._runtime

    @property
    def docker_client(self):
        """
        Returns:
            The docker client
        """
        return self._docker_client

    @classmethod
    def get_runtime(cls, code):
        """
        Build Runtime from code data

        Args:
            code: an instance of code class

        Returns:
            an instance of the right runtime
        """
        if code.lang == Lang.RUST:
            return Rust(code.version)

        elif code.lang == Lang.GO:
            return Go(code.version)

        elif code.lang == Lang.PHP:
            return PHP(code.version)

        elif code.lang == Lang.PYTHON:
            return Python(code.version)

        elif code.lang == Lang.CPLUS:
            return Cplus(code.version)

        elif code.lang == Lang.C:
            return C(code.version)

        elif code.lang == Lang.JAVA:
            if "main_class" in code.meta.keys():
                main_class = code.meta["main_class"]
            else:
                main_class = "Main"

            return Java(code.version, main_class)

        elif code.lang == Lang.RUBY:
            return Ruby(code.version)

        else:
            raise Exception("Invalid language {}".format(self._code.lang.value))



# MIT License
#
# Copyright (c) 2021 Clivern
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from .code import Code
from .lang import Lang
from .runtime.docker.c import C
from .runtime.docker.go import Go
from .runtime.docker.php import PHP
from .runtime.docker.rust import Rust
from .runtime.docker.ruby import Ruby
from .runtime.docker.java import Java
from .runtime.docker.cplus import Cplus
from .runtime.docker.python import Python
from .runtime.docker.engine import Engine
from .exception.invalid_runtime_version import InvalidRuntimeVersion


class Runner():
    """Factory Class for Runtime engine and Programming Languages Supported"""

    @classmethod
    def docker(cls, code, docker_client=None):
        """
        Get an instance of docker engine

        Args:
            code: an instance of Code class with the code snippet, language and version
            docker_client: an instance of docker client

        Returns:
            an instance of docker engine
        """
        if not code or not isinstance(code, Code):
            raise Exception("Code of type Code must be provided!")

        return Engine(code, docker_client)

    @classmethod
    def go(cls, code, version, id=None, meta={}):
        """
        Get an instance of go runtime

        Args:
            code: the code snippet to run
            version: the language version
            id: code item uuid
            meta: meta data that may be needed for the runtime

        Returns:
            an instance of Code class
        """
        if version not in Go().versions.keys():
            raise InvalidRuntimeVersion("Invalid version {} for runtime {}".format(
                version,
                Lang.GO.value
            ))

        return Code(code, Lang.GO, version, id, meta)

    @classmethod
    def php(cls, code, version, id=None, meta={}):
        """
        Get an instance of php runtime

        Args:
            code: the code snippet to run
            version: the language version
            id: code item uuid
            meta: meta data that may be needed for the runtime

        Returns:
            an instance of Code class
        """
        if version not in PHP().versions.keys():
            raise InvalidRuntimeVersion("Invalid version {} for runtime {}".format(
                version,
                Lang.PHP.value
            ))

        return Code(code, Lang.PHP, version, id, meta)

    @classmethod
    def ruby(cls, code, version, id=None, meta={}):
        """
        Get an instance of ruby runtime

        Args:
            code: the code snippet to run
            version: the language version
            id: code item uuid
            meta: meta data that may be needed for the runtime

        Returns:
            an instance of Code class
        """
        if version not in Ruby().versions.keys():
            raise InvalidRuntimeVersion("Invalid version {} for runtime {}".format(
                version,
                Lang.RUBY.value
            ))

        return Code(code, Lang.RUBY, version, id, meta)

    @classmethod
    def java(cls, code, version, id=None, meta={}):
        """
        Get an instance of java runtime

        Args:
            code: the code snippet to run
            version: the language version
            id: code item uuid
            meta: meta data that may be needed for the runtime

        Returns:
            an instance of Code class
        """
        if version not in Java().versions.keys():
            raise InvalidRuntimeVersion("Invalid version {} for runtime {}".format(
                version,
                Lang.JAVA.value
            ))

        return Code(code, Lang.JAVA, version, id, meta)

    @classmethod
    def python(cls, code, version, id=None, meta={}):
        """
        Get an instance of python runtime

        Args:
            code: the code snippet to run
            version: the language version
            id: code item uuid
            meta: meta data that may be needed for the runtime

        Returns:
            an instance of Code class
        """
        if version not in Python().versions.keys():
            raise InvalidRuntimeVersion("Invalid version {} for runtime {}".format(
                version,
                Lang.PYTHON.value
            ))

        return Code(code, Lang.PYTHON, version, id, meta)

    @classmethod
    def rust(cls, code, version, id=None, meta={}):
        """
        Get an instance of rust runtime

        Args:
            code: the code snippet to run
            version: the language version
            id: code item uuid
            meta: meta data that may be needed for the runtime

        Returns:
            an instance of Code class
        """
        if version not in Rust().versions.keys():
            raise InvalidRuntimeVersion("Invalid version {} for runtime {}".format(
                version,
                Lang.RUST.value
            ))

        return Code(code, Lang.RUST, version, id, meta)

    @classmethod
    def c(cls, code, version, id=None, meta={}):
        """
        Get an instance of C runtime

        Args:
            code: the code snippet to run
            version: the language version
            id: code item uuid
            meta: meta data that may be needed for the runtime

        Returns:
            an instance of Code class
        """
        if version not in C().versions.keys():
            raise InvalidRuntimeVersion("Invalid version {} for runtime {}".format(
                version,
                Lang.C.value
            ))

        return Code(code, Lang.C, version, id, meta)

    @classmethod
    def cplus(cls, code, version, id=None, meta={}):
        """
        Get an instance of C++ runtime

        Args:
            code: the code snippet to run
            version: the language version
            id: code item uuid
            meta: meta data that may be needed for the runtime

        Returns:
            an instance of Code class
        """
        if version not in Cplus().versions.keys():
            raise InvalidRuntimeVersion("Invalid version {} for runtime {}".format(
                version,
                Lang.CPLUS.value
            ))

        return Code(code, Lang.CPLUS, version, id, meta)



# MIT License
#
# Copyright (c) 2021 Clivern
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import pytest

from pindo.runtime.docker.go import Go
from pindo.runtime.docker.java import Java
from pindo.runtime.docker.php import PHP
from pindo.runtime.docker.python import Python
from pindo.runtime.docker.ruby import Ruby
from pindo.runtime.docker.rust import Rust


def test_go_runtime():
    """Go Runtime Tests"""
    go_rt = Go("1.0.0")

    assert isinstance(go_rt, Go) == True
    assert go_rt.script != ""
    assert "#!/bin/bash" in go_rt.script
    assert isinstance(go_rt.versions, dict) == True
    assert len(go_rt.versions) > 0
    assert go_rt.image != ""
    assert go_rt.extension != ""
    assert go_rt.version == "1.0.0"


def test_java_runtime():
    """Java Runtime Tests"""
    java_rt = Java("1.0.0")

    assert isinstance(java_rt, Java) == True
    assert java_rt.script != ""
    assert "#!/bin/bash" in java_rt.script
    assert isinstance(java_rt.versions, dict) == True
    assert len(java_rt.versions) > 0
    assert java_rt.image != ""
    assert java_rt.extension != ""
    assert java_rt.version == "1.0.0"
    assert java_rt.main_class == "Run"


def test_php_runtime():
    """PHP Runtime Tests"""
    php_rt = PHP("1.0.0")

    assert isinstance(php_rt, PHP) == True
    assert php_rt.script != ""
    assert "#!/bin/bash" in php_rt.script
    assert isinstance(php_rt.versions, dict) == True
    assert len(php_rt.versions) > 0
    assert php_rt.image != ""
    assert php_rt.extension != ""
    assert php_rt.version == "1.0.0"


def test_python_runtime():
    """Python Runtime Tests"""
    python_rt = Python("1.0.0")

    assert isinstance(python_rt, Python) == True
    assert python_rt.script != ""
    assert "#!/bin/bash" in python_rt.script
    assert isinstance(python_rt.versions, dict) == True
    assert len(python_rt.versions) > 0
    assert python_rt.image != ""
    assert python_rt.extension != ""
    assert python_rt.version == "1.0.0"


def test_ruby_runtime():
    """Ruby Runtime Tests"""
    ruby_rt = Ruby("1.0.0")

    assert isinstance(ruby_rt, Ruby) == True
    assert ruby_rt.script != ""
    assert "#!/bin/bash" in ruby_rt.script
    assert isinstance(ruby_rt.versions, dict) == True
    assert len(ruby_rt.versions) > 0
    assert ruby_rt.image != ""
    assert ruby_rt.extension != ""
    assert ruby_rt.version == "1.0.0"


def test_rust_runtime():
    """Rust Runtime Tests"""
    rust_rt = Rust("1.0.0")

    assert isinstance(rust_rt, Rust) == True
    assert rust_rt.script != ""
    assert "#!/bin/bash" in rust_rt.script
    assert isinstance(rust_rt.versions, dict) == True
    assert len(rust_rt.versions) > 0
    assert rust_rt.image != ""
    assert rust_rt.extension != ""
    assert rust_rt.version == "1.0.0"



# MIT License
#
# Copyright (c) 2021 Clivern
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import pytest

from pindo.code import Code
from pindo.lang import Lang


def test_code():
    """Code Tests"""
    code = Code("puts 'Hello World'", Lang.RUBY, "3.0.0")

    assert isinstance(code, Code) == True
    assert code.code == "puts 'Hello World'"
    assert code.version == "3.0.0"
    assert code.lang == Lang.RUBY
    assert code.lang.value == "ruby"
    assert code.id != ""

    new_code = Code.from_string(str(code))

    assert isinstance(new_code, Code) == True
    assert new_code.code == code.code
    assert new_code.version == code.version
    assert new_code.lang == code.lang
    assert new_code.lang.value == code.lang.value
    assert new_code.id == code.id



# MIT License
#
# Copyright (c) 2021 Clivern
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import pytest

from pindo.code import Code
from pindo.lang import Lang
from pindo.runner import Runner
from pindo.runtime.docker.engine import Engine
from pindo.exception.invalid_runtime_version import InvalidRuntimeVersion


def test_runner():
    """Runner Tests"""
    ruby_code = Runner.ruby("~~", "3.0.0")
    php_code = Runner.php("~~", "8.1")
    python_code = Runner.python("~~", "3.9")
    go_code = Runner.go("~~", "1.17")
    rust_code = Runner.rust("~~", "1.57.0")
    java_code = Runner.java("~~", "17.0")

    assert isinstance(ruby_code, Code) == True
    assert isinstance(php_code, Code) == True
    assert isinstance(python_code, Code) == True
    assert isinstance(go_code, Code) == True
    assert isinstance(rust_code, Code) == True
    assert isinstance(java_code, Code) == True

    # Invalid Version
    with pytest.raises(InvalidRuntimeVersion):
        Runner.ruby("~~", "1.0.0")

    with pytest.raises(InvalidRuntimeVersion):
        Runner.php("~~", "1.0.0")

    with pytest.raises(InvalidRuntimeVersion):
        Runner.python("~~", "1.0.0")

    with pytest.raises(InvalidRuntimeVersion):
        Runner.go("~~", "1.0.0")

    with pytest.raises(InvalidRuntimeVersion):
        Runner.rust("~~", "1.0.0")

    with pytest.raises(InvalidRuntimeVersion):
        Runner.java("~~", "1.0.0")

    # Invalid Code
    with pytest.raises(Exception):
        Runner.docker("", "~")

    assert isinstance(Runner.docker(ruby_code, "~"), Engine) == True
